# zenutils

Collection of simple utils. 

## Install

```
pip install zenutils
```

## Extra packages requires

- For python3.2 and python2.x, requires extra package: inspect2~=0.1.2
- For user who is using xxhash methods with hashutils, requires extra package: xxhash
- For user who is using sm3 methods with hashutils, requires extra package: sm3utils.
- If your python installation's hashlib already support sm3 hash method, you don't have to install sm3utils.
- xxhash and sm3utils are not put into this package's requirements, you need to install them by your self.

## Notice

- The hashutils' hash methods are different on different python installations. The list below is based on Python 3.10.6 x86_64 on windows. Mostly md5, sha1, sha224, sha256, sha384, sha512 methods are supported.
- The hashutils' DEFAULT_HASH_METHOD is sm3 and DEFAULT_PASSWORD_HASH_METHOD is ssm3, so if your python installation is not support sm3 hash method, you need to install sm3utils by yourself.

## Utils

1. zenutils.base64utils
    1. a85decode
    1. a85encode
    1. b16decode
    1. b16encode
    1. b32decode
    1. b32encode
    1. b32hexdecode
    1. b32hexencode
    1. b64decode
    1. b64encode
    1. b85decode
    1. b85encode
    1. decode
    1. decodebytes
    1. encode
    1. encodebytes
    1. standard_b64decode
    1. standard_b64encode
    1. urlsafe_b64decode
    1. urlsafe_b64encode
1. zenutils.baseutils
    1. Null
1. zenutils.cacheutils
    1. ReqIdCache
    1. cache
    1. get_cached_value
    1. simple_cache
1. zenutils.cipherutils
    1. Base64Encoder
    1. CipherBase
    1. DecryptFailed
    1. EncoderBase
    1. HexlifyEncoder
    1. IvCipher
    1. IvfCipher
    1. MappingCipher
    1. RawDataEncoder
    1. S12Cipher
    1. S1Cipher
    1. S2Cipher
    1. SafeBase64Encoder
    1. Utf8Encoder
1. zenutils.dateutils
    1. get_days
    1. get_months
    1. get_years
1. zenutils.dictutils
    1. HttpHeadersDict
    1. Object
    1. attrgetorset
    1. attrset
    1. change
    1. changes
    1. deep_merge
    1. diff
    1. fix_object
    1. ignore_none_item
    1. prefix_key
    1. select
    1. to_object
    1. touch
    1. update
1. zenutils.errorutils
    1. AccessDenied
    1. AccountDisabledError
    1. AccountLockedError
    1. AccountRemovedError
    1. AccountStatusError
    1. AccountTemporaryLockedError
    1. AnotherServiceError
    1. AppAuthFailed
    1. AuthError
    1. BadParameter
    1. BadParameterType
    1. BadResponseContent
    1. BadUserToken
    1. BizError
    1. BizErrorBase
    1. CacheError
    1. CaptchaOnlyAllowedOnce
    1. CaptchaRequired
    1. CaptchaValidateFailed
    1. CastFailedError
    1. CastToBooleanFailed
    1. CastToFloatFailed
    1. CastToIntegerFailed
    1. CastToNumbericFailed
    1. CastToStringFailed
    1. ClientLostError
    1. ConfigError
    1. DataError
    1. DatabaseError
    1. EventNotRegistered
    1. FormError
    1. HttpError
    1. InformalRequestError
    1. InformalResultPackage
    1. InformalResultPackage
    1. LogicError
    1. LoginRequired
    1. MessageQueueError
    1. MissingConfigItem
    1. MissingField
    1. MissingParameter
    1. NetworkError
    1. NoAccessPermissionError
    1. NoDeletePermissionError
    1. NoMatchingRouteFound
    1. NoPermissionError
    1. NoPermissionToCleanCacheError
    1. NoReadPermissionError
    1. NoUpstreamServerAvailabe
    1. NoWritePermissionError
    1. NotSupportedHttpMethod
    1. NotSupportedTypeToCast
    1. OK
    1. ParamError
    1. ParseJsonError
    1. PermissionError
    1. RepeatedlySubmitForm
    1. ReqeustForbidden
    1. ReqidDuplicateError
    1. RequestExpired
    1. SYSTEM_ERROR_CODE_MAPPING
    1. ServiceError
    1. StringTooLong
    1. StringTooShort
    1. SysError
    1. TargetNotFound
    1. TooLargeRequestError
    1. TsExpiredError
    1. TypeError
    1. UndefinedError
    1. UserDoesNotExist
    1. UserPasswordError
    1. ValueExceedsMaxLimit
    1. ValueLessThanMinLimit
    1. WrongFieldType
    1. WrongParameterType
    1. clean_language_name
    1. get_error_info
    1. get_language
    1. set_error_info
    1. set_language
1. zenutils.fsutils
    1. TemporaryFile
    1. copy
    1. expand
    1. file_content_replace
    1. filecopy
    1. first_exists_file
    1. get_application_config_filepath
    1. get_application_config_paths
    1. get_safe_filename
    1. get_size_deviation
    1. get_size_display
    1. get_swap_filename
    1. get_temp_workspace
    1. get_unit_size
    1. info
    1. mkdir
    1. move
    1. pathjoin
    1. readfile
    1. rename
    1. rm
    1. safe_write
    1. size_unit_names
    1. size_unit_upper_limit
    1. touch
    1. treecopy
    1. write
1. zenutils.funcutils
    1. BunchCallable
    1. ChainableProxy
    1. call_with_inject
    1. chain
    1. classproperty
    1. get_all_builtin_exceptions
    1. get_builtins_dict
    1. get_class_name
    1. get_default_values
    1. get_inject_params
    1. get_method_help
    1. get_method_signature
    1. inspect
    1. is_a_class
    1. isclass
    1. mcall_with_inject
    1. signature
    1. try_again_on_error
1. zenutils.hashutils
    1. Base64ResultEncoder
    1. Blake2BHexlifyPasswordHash
    1. Blake2BPbkdf2PasswordHash
    1. Blake2BPbkdf2PasswordHashColon
    1. Blake2BSimplePasswordHash
    1. Blake2BSimpleSaltPasswordHash
    1. Blake2SHexlifyPasswordHash
    1. Blake2SPbkdf2PasswordHash
    1. Blake2SPbkdf2PasswordHashColon
    1. Blake2SSimplePasswordHash
    1. Blake2SSimpleSaltPasswordHash
    1. DigestResultEncoder
    1. HexlifyPasswordHashBase
    1. HexlifyResultEncoder
    1. Md5HexlifyPasswordHash
    1. Md5Pbkdf2PasswordHash
    1. Md5Pbkdf2PasswordHashColon
    1. Md5SimplePasswordHash
    1. Md5SimpleSaltPasswordHash
    1. PasswordHashMethodBase
    1. PasswordHashMethodNotSupportError
    1. Pbkdf2PasswordHashBase
    1. ResultEncoderBase
    1. Sha1HexlifyPasswordHash
    1. Sha1Pbkdf2PasswordHash
    1. Sha1Pbkdf2PasswordHashColon
    1. Sha1SimplePasswordHash
    1. Sha1SimpleSaltPasswordHash
    1. Sha224HexlifyPasswordHash
    1. Sha224Pbkdf2PasswordHash
    1. Sha224Pbkdf2PasswordHashColon
    1. Sha224SimplePasswordHash
    1. Sha224SimpleSaltPasswordHash
    1. Sha256HexlifyPasswordHash
    1. Sha256Pbkdf2PasswordHash
    1. Sha256Pbkdf2PasswordHashColon
    1. Sha256SimplePasswordHash
    1. Sha256SimpleSaltPasswordHash
    1. Sha384HexlifyPasswordHash
    1. Sha384Pbkdf2PasswordHash
    1. Sha384Pbkdf2PasswordHashColon
    1. Sha384SimplePasswordHash
    1. Sha384SimpleSaltPasswordHash
    1. Sha3_224HexlifyPasswordHash
    1. Sha3_224Pbkdf2PasswordHash
    1. Sha3_224Pbkdf2PasswordHashColon
    1. Sha3_224SimplePasswordHash
    1. Sha3_224SimpleSaltPasswordHash
    1. Sha3_256HexlifyPasswordHash
    1. Sha3_256Pbkdf2PasswordHash
    1. Sha3_256Pbkdf2PasswordHashColon
    1. Sha3_256SimplePasswordHash
    1. Sha3_256SimpleSaltPasswordHash
    1. Sha3_384HexlifyPasswordHash
    1. Sha3_384Pbkdf2PasswordHash
    1. Sha3_384Pbkdf2PasswordHashColon
    1. Sha3_384SimplePasswordHash
    1. Sha3_384SimpleSaltPasswordHash
    1. Sha3_512HexlifyPasswordHash
    1. Sha3_512Pbkdf2PasswordHash
    1. Sha3_512Pbkdf2PasswordHashColon
    1. Sha3_512SimplePasswordHash
    1. Sha3_512SimpleSaltPasswordHash
    1. Sha512HexlifyPasswordHash
    1. Sha512Pbkdf2PasswordHash
    1. Sha512Pbkdf2PasswordHashColon
    1. Sha512SimplePasswordHash
    1. Sha512SimpleSaltPasswordHash
    1. ShaHexlifyPasswordHash
    1. ShaPbkdf2PasswordHash
    1. ShaPbkdf2PasswordHashColon
    1. ShaSimplePasswordHash
    1. ShaSimpleSaltPasswordHash
    1. SimplePasswordHashBase
    1. SimpleSaltPasswordHashBase
    1. Sm3HexlifyPasswordHash
    1. Sm3Pbkdf2PasswordHash
    1. Sm3Pbkdf2PasswordHashColon
    1. Sm3SimplePasswordHash
    1. Sm3SimpleSaltPasswordHash
    1. Xxh128HexlifyPasswordHash
    1. Xxh128Pbkdf2PasswordHash
    1. Xxh128Pbkdf2PasswordHashColon
    1. Xxh128SimplePasswordHash
    1. Xxh128SimpleSaltPasswordHash
    1. Xxh32HexlifyPasswordHash
    1. Xxh32Pbkdf2PasswordHash
    1. Xxh32Pbkdf2PasswordHashColon
    1. Xxh32SimplePasswordHash
    1. Xxh32SimpleSaltPasswordHash
    1. Xxh64HexlifyPasswordHash
    1. Xxh64Pbkdf2PasswordHash
    1. Xxh64Pbkdf2PasswordHashColon
    1. Xxh64SimplePasswordHash
    1. Xxh64SimpleSaltPasswordHash
    1. algorithms_available
    1. get_blake2b
    1. get_blake2b_base64
    1. get_blake2b_digest
    1. get_blake2b_hexdigest
    1. get_blake2s
    1. get_blake2s_base64
    1. get_blake2s_digest
    1. get_blake2s_hexdigest
    1. get_file_blake2b
    1. get_file_blake2b_base64
    1. get_file_blake2b_digest
    1. get_file_blake2b_hexdigest
    1. get_file_blake2s
    1. get_file_blake2s_base64
    1. get_file_blake2s_digest
    1. get_file_blake2s_hexdigest
    1. get_file_hash
    1. get_file_hash_base64
    1. get_file_hash_hexdigest
    1. get_file_hash_result
    1. get_file_md5
    1. get_file_md5_base64
    1. get_file_md5_digest
    1. get_file_md5_hexdigest
    1. get_file_sha
    1. get_file_sha1
    1. get_file_sha1_base64
    1. get_file_sha1_digest
    1. get_file_sha1_hexdigest
    1. get_file_sha224
    1. get_file_sha224_base64
    1. get_file_sha224_digest
    1. get_file_sha224_hexdigest
    1. get_file_sha256
    1. get_file_sha256_base64
    1. get_file_sha256_digest
    1. get_file_sha256_hexdigest
    1. get_file_sha384
    1. get_file_sha384_base64
    1. get_file_sha384_digest
    1. get_file_sha384_hexdigest
    1. get_file_sha3_224
    1. get_file_sha3_224_base64
    1. get_file_sha3_224_digest
    1. get_file_sha3_224_hexdigest
    1. get_file_sha3_256
    1. get_file_sha3_256_base64
    1. get_file_sha3_256_digest
    1. get_file_sha3_256_hexdigest
    1. get_file_sha3_384
    1. get_file_sha3_384_base64
    1. get_file_sha3_384_digest
    1. get_file_sha3_384_hexdigest
    1. get_file_sha3_512
    1. get_file_sha3_512_base64
    1. get_file_sha3_512_digest
    1. get_file_sha3_512_hexdigest
    1. get_file_sha512
    1. get_file_sha512_base64
    1. get_file_sha512_digest
    1. get_file_sha512_hexdigest
    1. get_file_sha_base64
    1. get_file_sha_digest
    1. get_file_sha_hexdigest
    1. get_file_sm3
    1. get_file_sm3_base64
    1. get_file_sm3_digest
    1. get_file_sm3_hexdigest
    1. get_file_xxh128
    1. get_file_xxh128_base64
    1. get_file_xxh128_digest
    1. get_file_xxh128_hexdigest
    1. get_file_xxh32
    1. get_file_xxh32_base64
    1. get_file_xxh32_digest
    1. get_file_xxh32_hexdigest
    1. get_file_xxh64
    1. get_file_xxh64_base64
    1. get_file_xxh64_digest
    1. get_file_xxh64_hexdigest
    1. get_hash
    1. get_hash_base64
    1. get_hash_hexdigest
    1. get_hash_result
    1. get_md5
    1. get_md5_base64
    1. get_md5_digest
    1. get_md5_hexdigest
    1. get_password_hash
    1. get_password_hash_methods
    1. get_pbkdf2_blake2b
    1. get_pbkdf2_blake2s
    1. get_pbkdf2_hmac
    1. get_pbkdf2_md5
    1. get_pbkdf2_sha
    1. get_pbkdf2_sha1
    1. get_pbkdf2_sha224
    1. get_pbkdf2_sha256
    1. get_pbkdf2_sha384
    1. get_pbkdf2_sha3_224
    1. get_pbkdf2_sha3_256
    1. get_pbkdf2_sha3_384
    1. get_pbkdf2_sha3_512
    1. get_pbkdf2_sha512
    1. get_pbkdf2_sm3
    1. get_pbkdf2_xxh128
    1. get_pbkdf2_xxh32
    1. get_pbkdf2_xxh64
    1. get_salted_hash_base64
    1. get_sha
    1. get_sha1
    1. get_sha1_base64
    1. get_sha1_digest
    1. get_sha1_hexdigest
    1. get_sha224
    1. get_sha224_base64
    1. get_sha224_digest
    1. get_sha224_hexdigest
    1. get_sha256
    1. get_sha256_base64
    1. get_sha256_digest
    1. get_sha256_hexdigest
    1. get_sha384
    1. get_sha384_base64
    1. get_sha384_digest
    1. get_sha384_hexdigest
    1. get_sha3_224
    1. get_sha3_224_base64
    1. get_sha3_224_digest
    1. get_sha3_224_hexdigest
    1. get_sha3_256
    1. get_sha3_256_base64
    1. get_sha3_256_digest
    1. get_sha3_256_hexdigest
    1. get_sha3_384
    1. get_sha3_384_base64
    1. get_sha3_384_digest
    1. get_sha3_384_hexdigest
    1. get_sha3_512
    1. get_sha3_512_base64
    1. get_sha3_512_digest
    1. get_sha3_512_hexdigest
    1. get_sha512
    1. get_sha512_base64
    1. get_sha512_digest
    1. get_sha512_hexdigest
    1. get_sha_base64
    1. get_sha_digest
    1. get_sha_hexdigest
    1. get_sm3
    1. get_sm3_base64
    1. get_sm3_digest
    1. get_sm3_hexdigest
    1. get_xxh128
    1. get_xxh128_base64
    1. get_xxh128_digest
    1. get_xxh128_hexdigest
    1. get_xxh32
    1. get_xxh32_base64
    1. get_xxh32_digest
    1. get_xxh32_hexdigest
    1. get_xxh64
    1. get_xxh64_base64
    1. get_xxh64_digest
    1. get_xxh64_hexdigest
    1. is_the_same_hash_method
    1. method_load
    1. new
    1. pbkdf2_hmac
    1. register_hexlify_password_hash
    1. register_password_hash_method
    1. register_pbkdf2_password_hash
    1. register_simple_password_hash
    1. register_simple_salt_password_hash
    1. setup_hash_method_loader
    1. validate_password_hash
    1. validate_pbkdf2_blake2b
    1. validate_pbkdf2_blake2s
    1. validate_pbkdf2_hmac
    1. validate_pbkdf2_md5
    1. validate_pbkdf2_sha
    1. validate_pbkdf2_sha1
    1. validate_pbkdf2_sha224
    1. validate_pbkdf2_sha256
    1. validate_pbkdf2_sha384
    1. validate_pbkdf2_sha3_224
    1. validate_pbkdf2_sha3_256
    1. validate_pbkdf2_sha3_384
    1. validate_pbkdf2_sha3_512
    1. validate_pbkdf2_sha512
    1. validate_pbkdf2_sm3
    1. validate_pbkdf2_xxh128
    1. validate_pbkdf2_xxh32
    1. validate_pbkdf2_xxh64
1. zenutils.httputils
    1. download
    1. get_sitename
    1. get_url_filename
    1. get_url_save_path
    1. get_urlinfo
    1. urlparse
1. zenutils.importutils
    1. get_caller_globals
    1. get_caller_locals
    1. import_from_string
    1. import_module
1. zenutils.jsonutils
    1. SimpleJsonEncoder
    1. make_simple_json_encoder
    1. register_global_encoder
    1. simple_json_dumps
1. zenutils.listutils
    1. append_new
    1. chunk
    1. clean_none
    1. compare
    1. compare_execute
    1. first
    1. group
    1. ignore_none_element
    1. int_list_to_bytes
    1. is_ordered
    1. list2dict
    1. pad
    1. replace
    1. topological_sort
    1. topological_test
    1. unique
1. zenutils.logutils
    1. get_console_handler
    1. get_file_handler
    1. get_simple_config
    1. setup
1. zenutils.nameutils
    1. get_last_names
    1. get_random_name
    1. get_suggest_first_names
    1. guess_lastname
    1. guess_surname
1. zenutils.numericutils
    1. _infinity
    1. binary_decompose
    1. bytes2ints
    1. decimal_change_base
    1. float_split
    1. from_bytes
    1. get_float_part
    1. infinity
    1. int2bytes
    1. ints2bytes
    1. is_infinity
    1. ninfinity
    1. pinfinity
1. zenutils.packutils
    1. AbstractResultPacker
    1. RcmPacker
1. zenutils.perfutils
    1. timeit
1. zenutils.randomutils
    1. Lcg31Random
    1. Random
    1. UuidGenerator
    1. choices
    1. get_password_seed32
    1. uuid1
    1. uuid3
    1. uuid4
    1. uuid5
1. zenutils.serviceutils
    1. DebugService
    1. ServiceBase
1. zenutils.sixutils
    1. BASESTRING_TYPES
    1. BYTES
    1. BYTES_TYPE
    1. INT_TO_BYTES
    1. NUMERIC_TYPES
    1. PY2
    1. PY3
    1. STR_TYPE
    1. TEXT
    1. bchar
    1. bstr_to_array
    1. bytes_to_array
    1. create_new_class
    1. default_encoding
    1. default_encodings
    1. force_bytes
    1. force_text
    1. unicode
1. zenutils.socketserverutils
    1. NStreamExchangeProtocolBase
    1. ServerEngineBase
    1. ServerHandle
1. zenutils.strutils
    1. BAI
    1. BASE64_CHARS
    1. HEXLIFY_CHARS
    1. QIAN
    1. SHI
    1. StrUtils
    1. URLSAFEB64_CHARS
    1. WAN
    1. YI
    1. binarify
    1. bytes2ints
    1. camel
    1. captital_number
    1. char_force_to_int
    1. chunk
    1. clean
    1. combinations
    1. combinations2
    1. decodable
    1. default_cn_digits
    1. default_cn_float_places
    1. default_cn_negative
    1. default_cn_places
    1. default_cn_yuan
    1. default_encoding
    1. default_encodings
    1. default_quotes
    1. default_random_string_choices
    1. do_clean
    1. encodable
    1. force_float
    1. force_int
    1. force_numberic
    1. force_type_to
    1. format_with_mapping
    1. get_all_substrings
    1. get_base64image
    1. get_image_bytes
    1. html_element_css_append
    1. int2bytes
    1. ints2bytes
    1. is_base64_decodable
    1. is_chinese_character
    1. is_hex_digits
    1. is_str_composed_by_the_choices
    1. is_unhexlifiable
    1. is_urlsafeb64_decodable
    1. is_uuid
    1. join_lines
    1. no_mapping
    1. none_to_empty_string
    1. parse_base64image
    1. random_string
    1. remove_prefix
    1. remove_suffix
    1. reverse
    1. simple_cn_yuan
    1. simplesplit
    1. smart_get_binary_data
    1. split
    1. split2
    1. str_composed_by
    1. stringlist_append
    1. strip_string
    1. substrings
    1. text_display_length
    1. text_display_shorten
    1. unbinarify
    1. unquote
    1. wholestrip
1. zenutils.sysutils
    1. default_timeout_kill
    1. execute_script
    1. get_current_thread_id
    1. get_node_ip
    1. get_random_script_name
    1. get_worker_id
    1. psutil_timeout_kill
1. zenutils.threadutils
    1. ConcurrentLimitJobQueue
    1. Counter
    1. Future
    1. JobExecuteTimeout
    1. JobQueue
    1. JobResultNotSet
    1. LoopIdle
    1. Service
    1. ServiceStop
    1. ServiceTerminate
    1. SimpleConsumer
    1. SimpleProducer
    1. SimpleProducerConsumerServer
    1. SimpleServer
    1. StartOnTerminatedService
1. zenutils.treeutils
    1. SimpleRouterTree
    1. build_tree
    1. print_tree
    1. print_tree_callback
    1. tree_walk
1. zenutils.typingutils
    1. Number
    1. STRING_ENCODINGS
    1. register_global_caster
    1. smart_cast
1. zenutils.xmlrpcutils
    1. SimpleAuthMixin
    1. SimpleAuthSafeTransport
    1. SimpleAuthTransport

## Compatibility

Test passed with python versions:

1. Python 2.7 passed
1. Python 3.2 passed
1. Python 3.3 passed
1. Python 3.4 passed
1. Python 3.5 passed
1. Python 3.7 passed
1. Python 3.8 passed
1. Python 3.9 passed
1. Python 3.10 passed
1. Python 3.11 passed

## Release

### v0.1.0

- First release.

### v0.2.0

- Add treeutils.SimpleRouterTree.
- Add randomutils.HashPrng.
- Add hashutils.get_password_hash and hashutils.validate_password_hash.
- Add dictutils.HttpHeadersDict.
- Add sysutils.get_node_ip.

### v0.3.1

- Add funcutils.retry.
- Fix hashutils.validate_password_hash problem.

### v0.3.2

- Add sm3 hash support in hashutils.
- Add xxhash hash support in hashutils.
- Export hashutils.pbkdf2_hmac to work with your self defined hash methods.
- Fix problem in sysutils.get_random_script_name on windows.
- Fix path string problem in tests.test_httputils on windows.

### v0.3.3

- Fix funcutils.isclass can not detect classes with metaclass. Use inspect.isclass instead.
- Add cacheutils.cache.

### v0.3.5

- Change default log file path from `pwd`/app.log to `pwd`/logs/app.log.
- Add fsutils.get_swap_filename.
- Add fsutils.safe_write.
- Add fsutils.get_safe_filename.

### v0.3.6

- Add sixutils.create_new_class.
- Fix hashutils problem in python3.3 and below.
- Extend numericutils.int2bytes as int.to_bytes.

### V0.3.7

- Add randomutils.Lcg31Random.
- Add randomutils.get_password_seed32.

### v0.3.8

- Fix force_text in handling NON-STR type data.

### v0.3.9

- Add delete_script parameter in fsutils.execute_script.

### v0.3.12

- dictutils.Object add select method.
- Add errorutils.
- Add packutils.
- Add perfutils.
- Add serviceutils.

### v0.3.15

- Add serviceutils.ServerEngineBase.

### v0.3.16

- Rename serviceutils.ServerEngineBase to socketserverutils.ServerEngineBase.
- Add socketserverutils.NStreamExchangeProtocolBase.
- Add ServerHandle.

### v0.3.19

- Fix NStreamExchangeProtocolBase makefile buffering problem for py2.7.
- Fix log format problem in old version python.
- Add more exception classes in errorutils.
- Add dateutils.
- Add errorutils.AuthenticationRequired.

### v0.3.23

- Fix logger.error missing msg problem.
- Add threadutils.ConcurrentLimitJobQueue.

### v0.4.8

- Fix *args problem in call_with_inject.
- Change serviceutils.ServiceBase.register_to.
- Add cacheutils.ReqIdCache.
- Add xmlrpcutils.SimpleAuthTransport.
- Add xmlrpcutils.SimpleAuthSafeTransport.
- Add threadutils.JobQueue.
- Add threadutils.Future.
- Add funcutils.get_method_help.
- Add funcutils.get_method_signature.

### v0.4.9

- Add cacheutils.simple_cache.

### v0.4.10

- Fix readme document.
- Fix pylint warnings.
- Unit test problems fix.

### v0.5.0

- Add sixutils.unicode.
